package com.jiuqi.util;

package .tmp;

import com.jiuqi.util.DuplicationException;
import com.jiuqi.util.IntValue;
import java.util.ArrayList;
import java.util.Comparator;

public class RecIndex {
  protected boolean[] descfields;
  
  public static final int lengthCompare(Object[] keys1, Object[] keys2, int length) {
    int ml = keys1.length;
    if (ml > keys2.length)
      ml = keys2.length; 
    if (ml > length)
      ml = length; 
    for (int i = 0; i < ml; i++) {
      int r = compare(keys1[i], keys2[i]);
      if (r != 0)
        return r; 
    } 
    if (ml >= length)
      return 0; 
    return keys1.length - keys2.length;
  }
  
  public static final int compare(Object rec1, Object rec2) {
    if (rec1 instanceof Object[] && rec2 instanceof Object[]) {
      Object[] oa1 = (Object[])rec1;
      Object[] oa2 = (Object[])rec2;
      int ml = oa1.length;
      if (ml > oa2.length)
        ml = oa2.length; 
      for (int i = 0; i < ml; i++) {
        int r = compare(oa1[i], oa2[i]);
        if (r != 0)
          return r; 
      } 
      return oa1.length - oa2.length;
    } 
    if (rec1 == null || rec2 == null)
      return ((rec1 == null) ? 0 : 1) - ((rec2 == null) ? 0 : 1); 
    if (rec1.equals(rec2))
      return 0; 
    try {
      if (rec1 instanceof Comparable)
        return ((Comparable<Object>)rec1).compareTo(rec2); 
      if (rec2 instanceof Comparable)
        return ((Comparable<Object>)rec2).compareTo(rec1); 
      return (((Boolean)rec1).booleanValue() ? 1 : 0) - (((Boolean)rec2).booleanValue() ? 1 : 0);
    } catch (ClassCastException classCastException) {
      return 0;
    } 
  }
  
  public static final boolean matchKey(Object pattern, Object key) {
    if (pattern == null)
      return true; 
    if (key == null)
      return false; 
    if (pattern.equals(key))
      return true; 
    if (pattern instanceof Object[] && key instanceof Object[]) {
      int len = ((Object[])pattern).length;
      if (len != ((Object[])key).length)
        return false; 
      for (int i = 0; i < len; i++) {
        if (((Object[])pattern)[i] != null && !((Object[])pattern)[i].equals(((Object[])key)[i]))
          return false; 
      } 
      return true;
    } 
    return false;
  }
  
  protected int lastFound = -1;
  
  protected ArrayList keys = new ArrayList(128);
  
  protected ArrayList values = new ArrayList(128);
  
  private Comparator keyComparator;
  
  public void setDescFields(boolean[] value) {
    this.descfields = value;
  }
  
  public void setCapacity(int capacity) {
    this.keys.ensureCapacity(capacity);
    this.values.ensureCapacity(capacity);
  }
  
  public void setKeyComparator(Comparator comparator) {
    this.keyComparator = comparator;
  }
  
  public final int compareKey(Object rec1, Object rec2) {
    if (this.keyComparator != null)
      return this.keyComparator.compare(rec1, rec2); 
    if (this.descfields == null)
      return compare(rec1, rec2); 
    if (rec1 instanceof Object[] && rec2 instanceof Object[]) {
      Object[] oa1 = (Object[])rec1;
      Object[] oa2 = (Object[])rec2;
      int ml = oa1.length;
      if (ml < oa2.length)
        ml = oa2.length; 
      for (int i = 0; i < ml; i++) {
        int j = compare(oa1[i], oa2[i]);
        if (j != 0) {
          if (this.descfields[i])
            j = -j; 
          return j;
        } 
      } 
      return (oa1.length > ml) ? 1 : ((oa2.length > ml) ? -1 : 0);
    } 
    int ret = compare(rec1, rec2);
    if (this.descfields[0])
      ret = -ret; 
    return ret;
  }
  
  public void clear() {
    this.keys.clear();
    this.values.clear();
    this.lastFound = -1;
  }
  
  public final int size() {
    return this.keys.size();
  }
  
  public final Object getKey(int index) {
    return this.keys.get(index);
  }
  
  public final Object getValue(int index) {
    return this.values.get(index);
  }
  
  public final void setValue(int index, Object value) {
    this.values.set(index, value);
  }
  
  public final int search(Object key) {
    if (this.lastFound >= 0 && compareKey(key, this.keys.get(this.lastFound)) == 0)
      return this.lastFound; 
    int sb = 0;
    int se = this.keys.size() - 1;
    int flag = -1;
    while (sb <= se) {
      int sp = sb + se >> 1;
      flag = compareKey(key, this.keys.get(sp));
      if (flag > 0) {
        sb = sp + 1;
        continue;
      } 
      if (flag < 0) {
        se = sp - 1;
        continue;
      } 
      sb = sp;
      break;
    } 
    return (flag == 0) ? sb : (-sb - 1);
  }
  
  public final int searchfirst(Object key) {
    int sb = 0;
    int se = this.keys.size() - 1;
    int flag = -1;
    while (sb <= se) {
      int sp = sb + se >> 1;
      flag = compareKey(key, this.keys.get(sp));
      if (flag > 0) {
        sb = sp + 1;
        continue;
      } 
      if (flag < 0) {
        se = sp - 1;
        continue;
      } 
      se = sp;
      if (sb == sp)
        break; 
    } 
    return (flag == 0) ? sb : (-sb - 1);
  }
  
  public final int searchlast(Object key) {
    int sb = 0;
    int se = this.keys.size() - 1;
    int flag = -1;
    while (sb <= se) {
      int sp = sb + se + 1 >> 1;
      flag = compareKey(key, this.keys.get(sp));
      if (flag > 0) {
        sb = sp + 1;
        continue;
      } 
      if (flag < 0) {
        se = sp - 1;
        continue;
      } 
      sb = sp;
      if (se == sp)
        break; 
    } 
    return (flag == 0) ? sb : (-sb - 1);
  }
  
  public final int partlySearch(Object[] partKey, IntValue count) {
    int sb = 0;
    int lastEqual = 0;
    int se = this.keys.size() - 1;
    int lastLarger = se;
    int flag = -1;
    while (sb <= se) {
      int sp = sb + se >> 1;
      flag = lengthCompare(partKey, this.keys.get(sp), partKey.length);
      if (flag > 0) {
        sb = sp + 1;
        continue;
      } 
      if (flag < 0) {
        se = sp - 1;
        lastLarger = se;
        continue;
      } 
      lastEqual = sp;
      se = sp;
      if (sb == sp)
        break; 
    } 
    if (flag != 0) {
      count.value = 0;
      return -sb - 1;
    } 
    se = lastLarger;
    lastLarger = sb;
    sb = lastEqual;
    while (sb <= se) {
      int sp = sb + se + 1 >> 1;
      flag = lengthCompare(partKey, this.keys.get(sp), partKey.length);
      if (flag > 0) {
        sb = sp + 1;
        continue;
      } 
      if (flag < 0) {
        se = sp - 1;
        continue;
      } 
      sb = sp;
      if (se == sp)
        break; 
    } 
    count.value = sb - lastLarger + 1;
    return lastLarger;
  }
  
  public final boolean haskey(Object key) {
    return (search(key) >= 0);
  }
  
  public final int indexOf(Object key) {
    int index = search(key);
    return (index < 0) ? -1 : index;
  }
  
  public final Object findkey(Object key) {
    int index = search(key);
    if (index < 0)
      return null; 
    this.lastFound = index;
    return this.values.get(index);
  }
  
  public final int findfirstkey(Object key) {
    int index = searchfirst(key);
    if (index < 0)
      return -1; 
    return index;
  }
  
  public final int findlastkey(Object key) {
    int index = searchlast(key);
    if (index < 0)
      return -1; 
    return index;
  }
  
  public void add(Object key) throws DuplicationException {
    add(key, new Integer(this.keys.size()));
  }
  
  public void add(Object key, Object value) throws DuplicationException {
    int index = search(key);
    if (index >= 0) {
      if (this.values.get(index) == value)
        return; 
      throw new DuplicationException(("键值重复：" + key == null) ? null : key.toString());
    } 
    index = -index - 1;
    this.keys.add(index, key);
    this.values.add(index, value);
    this.lastFound = index;
  }
  
  public int addNew(Object key) {
    return addNew(key, new Integer(this.keys.size()));
  }
  
  public int addNew(Object key, Object value) {
    int index = search(key);
    if (index < 0) {
      index = -index - 1;
      this.keys.add(index, key);
      this.values.add(index, value);
      this.lastFound = index;
    } 
    return index;
  }
  
  public void remove(int index) {
    if (index < this.lastFound)
      this.lastFound--; 
    if (index == this.lastFound)
      this.lastFound = -1; 
    this.keys.remove(index);
    this.values.remove(index);
  }
  
  public boolean remove(Object key) {
    int index = search(key);
    if (index >= 0) {
      this.keys.remove(index);
      this.values.remove(index);
      if (index < this.lastFound)
        this.lastFound--; 
      if (index == this.lastFound)
        this.lastFound = -1; 
    } 
    return (index >= 0);
  }
  
  public final int addfirst(Object key) {
    return addfirst(key, new Integer(this.keys.size()));
  }
  
  public final int addfirst(Object key, Object value) {
    int index = searchfirst(key);
    if (index < 0)
      index = -index - 1; 
    this.keys.add(index, key);
    this.values.add(index, value);
    this.lastFound = index;
    return index;
  }
  
  public final boolean removefirst(Object key) {
    int index = searchfirst(key);
    if (index >= 0) {
      this.keys.remove(index);
      this.values.remove(index);
      if (index < this.lastFound)
        this.lastFound--; 
      if (index == this.lastFound)
        this.lastFound = -1; 
    } 
    return (index >= 0);
  }
  
  public final int addlast(Object key) {
    return addlast(key, new Integer(this.keys.size()));
  }
  
  public final int addlast(Object key, Object value) {
    int index = searchlast(key);
    if (index >= 0) {
      index++;
    } else {
      index = -index - 1;
    } 
    this.keys.add(index, key);
    this.values.add(index, value);
    this.lastFound = index;
    return index;
  }
  
  public final boolean removelast(Object key) {
    int index = searchlast(key);
    if (index >= 0) {
      this.keys.remove(index);
      this.values.remove(index);
      if (index < this.lastFound)
        this.lastFound--; 
      if (index == this.lastFound)
        this.lastFound = -1; 
    } 
    return (index >= 0);
  }
  
  public final int indexOfValue(Object value) {
    return this.values.indexOf(value);
  }
}
